require 'nadir/config'
require 'nadir/middleware/rack'
require 'nadir/notification'
require 'nadir/transport/http_async'
require 'nadir/version'

module Nadir
  extend self

  def config
    @config ||= Config.new
  end

  def configure
    yield config if block_given?
  end

  def logger
    config.logger
  end

  def notify(exception, params = {})
    return false unless config.validate

    notification = Notification.new exception, params

    transport.deliver notification.to_params

    true
  rescue => e
    logger.error "[Nadir] Internal error: #{e.inspect}"

    false
  end

  private

  def transport
    @_transport ||= Nadir::Transport::HTTPAsync.new
  end
end

require 'nadir/plugins/rails' if defined? Rails::Railtie
require 'nadir/plugins/rake' if defined? Rake::Application
