module Nadir
  module Middleware
    class Rack
      def initialize(app)
        @app = app
      end

      def call(env)
        begin
          response = @app.call(env)
        rescue Exception => exception
          begin
            request = ActionDispatch::Request.new env

            headers = request.headers.env.select{|k, _| k.in?(ActionDispatch::Http::Headers::CGI_VARIABLES) || k =~ /^HTTP_/}
            location = request.parameters.values_at('controller', 'action').compact.join('#')
            location = location.presence || "#{headers['REQUEST_METHOD']} #{headers['PATH_INFO']}"

            request_params = {
              headers: headers,
              params: request.parameters,
              remote_ip: request.remote_ip,
              user: {}
            }

            if request.env['warden'] && request.env['warden'].respond_to?(:user)
              request_params[:user][:id] = request.env['warden'].user&.id
            end

            Nadir.notify exception, location: location, request: request_params
          rescue => e
            Nadir.logger.error "[Nadir] Internal error: #{e.inspect}"
          end

          raise
        end

        response
      ensure
        # Nadir.clear_request_data
      end
    end
  end
end
